function best_rules = boost_rules(data, tries, rules, ...
					min_pod, max_far, overlap_window)

% Boost rules implements ADABoost on the rules found in combine_rules2. 
% ADABoost combines different hypotheses learned for the rules in an ensemble method. 
% boost_rules returns a struct of the best rules (ranked in order by their critical 
% success index) and the rule's weight based on performance across all storms

num_pos = data(1).num_pos;
num_neg = data(1).num_neg;
num_features = data(1).num_features;
storm_w = (ones(1, (num_pos+num_neg))/(num_pos+num_neg)); % initialize the weights for each example to 1
rule_err = 0;
num_rules = min(30, length(rules));
used_idxs = [0];

% for each hypothesis do:
for rule_idx = 1:num_rules
    
    % Find the rule that has the best performance on the data
    % best performance is defined as the highest critical success index
    [best, used_idxs] = get_best_rule(data, rules, storm_w, used_idxs);
    rule_err = 0; % initialize the rule error to zero
    
disp(['The rule is ', num2str(rule_idx)]);
    
    % STEP 1: Calculate the error for the rule
    
    % We need to iterate through each storm's feature's sax-ified data to
    % find the error for the rule
    for feature = 1:num_features
        if(~isempty(find(feature == best.features)))

            for storm = 1:(num_pos+num_neg)
            
               % if the storm is positive, get the discrete positive data for
               % the storm, if it's negative, get the discrete negative data.
               % Store the symbols for the feature trajectory in my_data
               if(storm <= num_pos)
                   my_data = data(storm).discrete_pos(feature).symbols;
               else
                   my_data = data(storm-num_pos).discrete_neg(feature).symbols;
               end

               % find all locations of the best rule's word in the vector
               locs = find_word(my_data, best.trie.seq);
            
               % if the word does not occur in the storm, then add to the
               % error
               if (isempty(locs))
                  rule_err = rule_err + storm_w(storm);
               end  
            end
        end
    end
 
    % STEP 2: Update the weights for the storms and the rules
    
    % We need to iterate through each storm's feature's sax-ified data to
    % update the weight for each storm based on the error
    for feature = 1:num_features
        if(~isempty(find(feature == best.features)))
            for storm = 1:(num_pos+num_neg)
            
                % if the storm is positive, get the discrete positive data for
                % the storm, if it's negative, get the discrete negative data.
                % Store the symbols for the feature trajectory in my_data
                if(storm <= num_pos)
                    my_data = data(storm).discrete_pos(feature).symbols;
                else
                   my_data = data(storm-num_pos).discrete_neg(feature).symbols;
                end

                % find all locations of the best rule's word in the vector
                locs = find_word(my_data, best.trie.seq);
            
                % if the word does occur in the storm, then add to the
                % the weight of the storm by the rule error
                if (isempty(locs))
                    storm_w(storm) = storm_w(storm) * ((1-rule_err)/rule_err);
                end  
             end
          end
      end

% normalize the weights for each storm so they fall between [0,1], so we
% optimize the weights by finding the absolute value of the normalized
% weight
if(~(isempty(find(storm_w < 0)) && isempty(find(storm_w > 1))))
    weight_mean = mean(storm_w);
    weight_std = std(storm_w);
    storm_w = abs((storm_w - weight_mean)/weight_std);
end

% Assign the best rule and its weight to the best_rules struct
    best_rules(rule_idx).rule = best;
    best_rules(rule_idx).weight = (1-rule_err)/rule_err;
end

