function [valid, rule] = is_valid_rule(rule1, rule2, min_pod, max_far, ...
                                        num_pos, num_neg, overlap_window)
% Determines if rule1 and rule2 concatenated is a rule that satisfies the
% POD-FARatio requirement. If so, then boo is set to true and the rule is
% set to the valid rule with all rule information. If not, then boo is set
% to false, and rule is left undeclared. 

epsilon = .0000000000001; % avoid divide by zero error
valid = false; % we need to flag the combined rule if it's legal
within_window = 10; % the rules must occur within 10 minutes of eachother

% Initialize the trie we're going to attempt a rule from
test_trie.seq = [rule1.trie.seq; rule2.trie.seq];
test_trie.seq_index = [rule1.trie.seq_index, rule2.trie.seq_index];
test_trie.pos_counts = 0;
test_trie.neg_counts = 0;
test_trie.traj_index = [];
test_trie.traj_labels = [];
test_trie.start_idxs = [];
occurs = 0;

% initialize rule
rule.features = [0];
rule.trie = test_trie;
rule.pod = 0;
rule.fa_rate = 0;
rule.fa_ratio = 0;
rule.csi = 0;
rule.csi_weight = 0;
%rule.chis = 0;
%rule.p_vals = 0;
rule.exist = 0;

% Create a matrix of the storm ids x storm labels in which each rule occurs
vec1 = [rule1.trie.traj_index' rule1.trie.traj_labels'];
vec2 = [rule2.trie.traj_index' rule2.trie.traj_labels'];

% Find the unique storms for rule1 and rule2
uniques1 = unique(vec1, 'rows'); 
uniques2 = unique(vec2, 'rows');

% Find indices into uniques1 of the storms the rules have in common
idxs = ismember(uniques1, uniques2, 'rows');
vals = uniques1(idxs,:);

        
% Create the trie for the new rule (rule1, rule2) if it's valid
for ind = 1:size(vals, 1)
    % Get the indices in each rule that share storm ind
    times1 = rule1.trie.start_idxs(:,(rule1.trie.traj_index == vals(ind, 1)));
    times2 = rule2.trie.start_idxs(:,(rule2.trie.traj_index == vals(ind, 1)));
    trie_times = zeros(size(times1,1) + size(times2,1), size(times1,2) * size(times2,2));
    times_idx = 1;
    
    % Find whether or not the rules occur in the same feature
    if(size(intersect(rule1.features, rule2.features),1) > 0)
        feat = 0; % They do occur in the same feature
    else
        feat = 1; % They don't occur in the same feature
    end
    
    % FIND TIME OF OCCURRENCE OF RULE
    gr_flag = 0;
    eq_flag = 0;
    for time = 1:size(times1,2)
        % Get the last time of a feature occurrence in rule1 and compare 
        % to the start times of rule2
        t = times1(size(times1,1),time);
        
        % Check that the times are greater than the overlap_window but fall
        % within the within_window, and those are the indices used
        ids_greater = find((times2(1,:)-t) >= overlap_window);
        ids_within = find((times2(1,:)-t) < within_window);

        % get the indices that are both greater and fall within the window
	ids_greater = ids_greater(find(ismember(ids_greater, ids_within) == 1));
        
        % Find the times in which the two rules occur simultaneously
        %ids_equal = find((times2(1,:)-t) == 0);
        ids_equal = [];  % means that none of the features can overlap
        
        % If the time difference between the two rules is larger than the
        % overlap_window, then the combined rule is valid
        if(~isempty(ids_greater))
            gr_flag = 1;
        end

        if(~isempty(ids_equal)) % if the two rules overlap at the same 
                                    % time, we need to verify that the
                                    % features are unique in order for the
                                    % rule to be valid
            if(feat == 1)
                eq_flag = 1;
                ids_eq = ids_equal;
            else
                % We still need to verify that rule 2 does not belong to
                % the same feature of rule 1 at the same time earlier in
                % the feature list
                
                % find all of the features that occur in both rule1 and
                % rule2
                val = intersect(rule1.features, rule2.features);
                
                % for each feature that occurs in both rules, find the
                % start_idxs for the feature within the rule. Then, find
                % the start_idxs for which rule1 comes before rule2
                for v = 1:length(val)
                    idxs1 = rule1.trie.start_idxs(find(rule1.features == val(v)),...
                        (rule1.trie.traj_index == vals(ind, 1)));
                    idxs2 = rule2.trie.start_idxs(find(rule2.features == val(v)),...
                        (rule2.trie.traj_index == vals(ind, 1)));
                    t1 = idxs1(size(idxs1,1), time);
                    ids_eq = find((idxs2(1,:)-t1) >= overlap_window);
                    if(~isempty(ids_eq))
                        eq_flag = 1;
                    else
                        eq_flag = 0;
                        break;
                    end
                end
            end
        end

        % Now build the start times for the rules
        if(~isempty(ids_greater) && gr_flag == 1)
           if(~isempty(ids_equal) && eq_flag == 1)
               % Get the unique ids of the indices into the start_idxs
               ids = unique([ids_greater, ids_eq]);
               starts = [(times1(:,time) * ones(1,length(ids))); ...
                          times2(:,ids)];
           else
	       starts = [(times1(:,time) * ones(1,length(ids_greater))); ...
								times2(:,ids_greater)];
           end
        elseif(~isempty(ids_equal) && eq_flag == 1)
            starts = [(times1(:,time) * ones(1,length(ids_eq))); ...
							   times2(:, ids_eq)];
        else 
            continue;
        end
        if(~isempty(starts))
            trie_times(:,times_idx:times_idx+size(starts,2)-1) = starts;
            times_idx = times_idx+size(starts,2);
        end
    end
    
    if(times_idx ~= 1)
        % Add start times to trie
        best_times = trie_times(:,1:(times_idx-1));
        test_trie.start_idxs = [test_trie.start_idxs, best_times];

        % Get the storm number
        strm_num = vals(ind,1);
        new_idxs = strm_num*(ones(1,size(best_times,2)));
        test_trie.traj_index = [test_trie.traj_index, new_idxs];

        % Get the storm label
        strm_label = vals(ind,2);
        new_labels = strm_label*(ones(1,size(best_times,2)));
        test_trie.traj_labels = [test_trie.traj_labels, new_labels];

        % Set flag to indicate that a rule was created
        occurs = 1;
    end
end

% If no trie was built for this rule, don't bother creating data tables.
if(occurs == 0)
    return;
end

% Count up the number of positive and negative storms this trie occurs in
neg_idxs = find(test_trie.traj_labels == 0);
pos_idxs = find(test_trie.traj_labels == 1);

test_trie.pos_counts = length(unique(test_trie.traj_index(pos_idxs)));
test_trie.neg_counts = length(unique(test_trie.traj_index(neg_idxs)));

% Build the contingency table for this trie
table = build_contingency_table(test_trie, num_pos, num_neg);

% Calculate the Probability of Detection attained by the trie
pod = table(1,1) / (table(1,1) + table(2,1) + epsilon);

% Calculate the False Alarm Ratio attained by the trie
fa_ratio = table(1,2) / (table(1,1) + table(1,2) + epsilon);

% Calculate the False Alarm Rate attained by the trie
fa_rate = table(1,2) / (table(2,2) + table(1,2) + epsilon);

% Calculate the Critical Success Index attained by the trie
csi = table(1,1) / (table(1,1) + table(1,2) + table(2,1) + epsilon); 

% Calculate the chi^2 value for this contingency table
%chis = calc_chisquared(table);
%p_vals = 1 - chi2pdf(chis, ones(1, length(chis)));

% if pod>min_pod and if far<max_far then we want to add the word to
% our list of rules
if (pod > min_pod && fa_ratio < max_far)
    rule.features = [rule1.features; rule2.features];
    rule.trie = test_trie;
    rule.pod = pod;
    rule.fa_rate = fa_rate;
    rule.fa_ratio = fa_ratio;
    rule.csi = csi;
    rule.csi_weight = 0;
    %rule.chis = chis;
    %rule.p_vals = p_vals;
    rule.exist = 1;

    valid = true;
end
