function plot_rule_combined(data, tries, storm_idx, storm_label, ...
    rules, rule_idx, path, varargin)
% Plot_rule_combined plots both the discretized rule and the real valued
% rule for a paticular storm side-by-side. Data contains all the storms and
% storm_index tells you which storm to print.  If storm label = 1, then it
% assumes the storm_index is for a positive storm and if storm_label = 0,
% then it assumes it is for a negative storm.  If the rule does not fire
% for this storm, it returns.

global feature_map;
global averaging_interval alphabet_size trie_word_size;

% Check to make sure this rule shows up in the given storm
rule = rules(rule_idx);

% fix this to come from the trie automatically
overlap_window = 1;
maximum_overlap = 10;

features = rule.features;
rule_depth = size(features,1);
feature = rule.features(1);
seq_index = rule.trie.seq_index(1);

invalid_rule = 0;
% see when this rule fires for this storm
if (rule_depth > 1)
    s = rule.trie.seq_index(1);
    trie = tries(rule.features(1)).trie(s);

    % start by seeing all times that it fired for this storm for the first
    % feature
    if (storm_label == 1)
        last_starts = trie.pos_starts{storm_idx};
    else
        last_starts = trie.neg_starts{storm_idx};
    end
    last_feature = rule.features(1);
    rule_starts{1} = last_starts;

    % now evaluate features 2 to rule depth
    for d = 2:rule_depth
        s = rule.trie.seq_index(d);
        feature = rule.features(d);
        trie = tries(feature).trie(s);
        if (storm_label == 1)
            starts = trie.pos_starts{storm_idx};
        else
            starts = trie.neg_starts{storm_idx};
        end

        % see if the intersection of the starts is within the specified
        % overlap window
        valid_starts = [];
        for k = 1:length(last_starts)
            diffs = starts - last_starts(k);

            if (last_feature == feature)
                seq_length = size(rule.trie.seq,2);
                idxs = find(diffs >= (overlap_window + seq_length - 1) ...
                    & diffs <= maximum_overlap);
            else
                idxs = find(diffs >= overlap_window & diffs <= maximum_overlap);
            end

            valid_starts = [valid_starts, starts(idxs)];
        end
        last_starts = unique(valid_starts);
        rule_starts{d} = last_starts;
        if (isempty(last_starts))
            invalid_rule = 1;
            break;
        end
        last_feature = feature;
    end
else
    % the rule is only depth 1 so just access the trie to get the
    % starting indices
    s = rule.trie.seq_index(1);    
    if (storm_label == 1)
        rule_starts{1} = tries(feature).trie(s).pos_starts{storm_idx};
    else
        rule_starts{1} = tries(feature).trie(s).neg_starts{storm_idx};
    end
end

if (invalid_rule)
    disp(['Rule ', num2str(rule_idx), ' does not fire for this storm']);
    return;
end


color = {'g', 'y', 'm', 'c', 'b', 'r', 'k', 'g', 'y'};
symbols = {'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i'};

count = 1; % Keeps track of the row-wise index within subplot

for f = 1:rule_depth
    % Set up the plot
    subplot(rule_depth, 2, count, 'align');
    set(gcf, 'Position', [0 0 1024 1024]);
    set(gcf, 'PaperPositionMode', 'auto');
    hold on;

    % extract the feature mean and standard deviation
    feat_mean = feature_map(features(f)).mean;
    feat_std = feature_map(features(f)).std;

    % Plot the guidelines for the normalization for each feature
    switch alphabet_size
        case 3, cutlines  = feat_mean + (feat_std * [-0.43 0.43]);
        case 4, cutlines  = feat_mean + (feat_std * [-0.67 0 0.67]);
        case 5, cutlines  = feat_mean + (feat_std * [-0.84 -0.25 0.25 0.84]);
        case 6, cutlines  = feat_mean + (feat_std * [-0.97 -0.43 0 0.43 0.97]);
        case 7, cutlines  = feat_mean + (feat_std * [-1.07 -0.57 -0.18 0.18 0.57 1.07]);
        case 8, cutlines  = feat_mean + (feat_std * [-1.15 -0.67 -0.32 0 0.32 0.67 1.15]);

        otherwise
            disp('Unknown alphabet_size - returning');
            return;
    end

    hold on;

    guidelines = repmat(cutlines', 1, 10);
    h = plot(guidelines', 'color', [0.4 0.4 0.4]);
    axis([1, 10, min(cutlines - feat_std), max(cutlines) + feat_std]);
    feature_axis = axis;
    hold on;

    % Get the sequence for this feature
    seq = rule.trie.seq(f,:);

    % Draw the segment and the symbol
    seg = map_to_values(seq, symbols, cutlines, feat_std);
    x_start = 1;
    x_end = 3;
    x_mid = 2;

    for s = 1:size(seg,2)
        plot([x_start:x_end], ones(1,3)*seg(s), 'color', color{seq(s)},  'linewidth', 4);
        text(x_mid, seg(s), symbols{seq(s)}, 'fontsize', 24);
        x_start = x_end + 1;
        x_end = x_end + 3;
        x_mid = x_start + (x_end - x_start)/2;
    end
    
    label_plot(data, rule, rule_idx, storm_idx, count, feature_map, ...
        features, f, storm_label, 'd');
    count = count + 1;

    % Plot the rule for the real-valued storm data
    subplot(rule_depth, 2, count, 'align');

    % show the entire history
    if(storm_label == 1)
      plot(data(storm_idx).pos(features(f),:), 'b');
    else
      plot(data(storm_idx).neg(features(f),:), 'b');
    end

    % Plot the occurences of the rule overlayed on the original data
    hold on;
    all_starts = rule_starts{f};
    start_time = ((all_starts(1) - 1) * averaging_interval) + 1;
    end_time = (size(seq,2) * averaging_interval) + start_time - 1;
    if(storm_label == 1)
      d = data(storm_idx).pos(features(f), start_time:end_time);
    else
      d = data(storm_idx).neg(features(f), start_time:end_time);
    end
    
    h = plot([start_time:end_time], d, 'r');
    set(h, 'LineWidth', 3);

    label_plot(data, rule, rule_idx, storm_idx, count, feature_map, ...
        features, f, storm_label, 'r');
    my_axis = axis;
    axis([my_axis(1), my_axis(2), feature_axis(3), feature_axis(4)]);
    count = count + 1;
end

% Save figure
fileName = [path, 'rule_', num2str(rule_idx), '.eps'];
print('-depsc ', fileName);
fileName = [path, 'rule_', num2str(rule_idx), '.png'];
print('-dpng ', fileName);
close;

%--------------------------------------------------------------------------------------------------------------------------------------------------------
%----------------Local Functions----------------------Local Functions----------------Local Functions----------------------Local Functions----------------
%--------------------------------------------------------------------------------------------------------------------------------------------------------

function label_axes(idx)
global feature_map;

ylabel(feature_map(idx).units);

function label_plot(data, rule, rule_idx, storm_idx, count, feature_map, ...
    features, f, storm_label, plot_label)
% Label the plot
if (storm_label == 1)
    name = data(storm_idx).pos_name;
else
    name = data(storm_idx).neg_name;
end

% remove the underscores as it interprets them as a latex subscript command
fix_idxs = regexp(name, '\_');
name(fix_idxs) = ' ';

% plot
if(f == 1)
    hold on;
    %[pod_str, err] = sprintf('%1.2f / %1.2f', rule.pod, rule.pod_cross_val);
    %[far_str, err] = sprintf('%1.2f / %1.2f', rule.fa_ratio, rule.fa_ratio_cross_val);
    %[csi_str, err] = sprintf('%1.2f / %1.2f', rule.csi, rule.csi_cross_val);

    [pod_str, err] = sprintf('%1.2f ', rule.pod);
    [far_str, err] = sprintf('%1.2f ', rule.fa_ratio);
    [csi_str, err] = sprintf('%1.2f ', rule.csi);

    
    if(storm_label == 1)
        %title({['Rule ', num2str(rule_idx), ' on storm ', ...
        %    name, ' has training/test ']; ['POD ', pod_str, ...
        %    ' FAR (ratio) of ', far_str, ' and CSI of ', ...
        %    csi_str]; ['Feature ', feature_map(features(f)).name]});
        title({['Rule ', num2str(rule_idx), ' on ', ...
            name, ' has training ']; ['POD ', pod_str, ...
            ' FAR (ratio) of ', far_str, ' and CSI of ', ...
            csi_str]; ['Feature ', feature_map(features(f)).name]});
        if (plot_label == 'r')
            xlabel('time');
        end
    else
        title({['Rule ', num2str(rule_idx), ' on ', ...
            name]; [' has training/test POD ', pod_str, ...
            ' and FAR (ratio) of ', far_str, ' and CSI of ', csi_str];...
            ['Feature ', feature_map(features(f)).name]});
    end
else
    if(plot_label == 'r')
        title(['Feature ', feature_map(features(f)).name]);
        xlabel('time');
    else
        title(['Feature ', feature_map(features(f)).name]);
    end
end

label_axes(features(f));

%%%% turn the symbols into points on the line
function seg = map_to_values(seq, symbols, cutlines, feat_std)
seg = [];

for s = 1:length(seq)
    switch seq(s)
        case 1, val = (cutlines(1) - (feat_std / 2));
        case 2, val = (cutlines(1) + (feat_std / 2));
        case 3, val = (cutlines(2) + (feat_std / 2));
        case 4, val = (cutlines(3) + (feat_std / 2));
        case 5, val = (cutlines(4) + (feat_std / 2));
        case 6, val = (cutlines(5) + (feat_std / 2));
        case 7, val = (cutlines(6) + (feat_std / 2));
        case 8, val = (cutlines(7) + (feat_std / 2));
        otherwise, disp('WARNING');
    end
    seg = [seg val];
end
