function rule = test_rule (rule, data, tries, alphabet_size, overlap_window)

% Test rule tests to see if the rule is valid for the given data and tries.
% If so, then it returns valid = 1 and the pod, far, and csi of this rule
% on the test_data/test_tries

if(rule.exist == 0)
    rule.pod_cross_val = 0;
    rule.fa_ratio_cross_cal = 0;
    rule.fa_rate_cross_val = 0;
    rule.csi_cross_val = 0;
    return;
end

epsilon = .000000000000000001; % avoid divide by zero

% Get the trie for this particular feature
trie = rule.trie;

% maximum overlap is 10 minutes - FIX this to not be hard-coded
maximum_overlap = 10;

% check to see if this rule fires in the test set
rule_depth = size(rule.features,1);

% setup some variables for the loop
feature = rule.features(1);
seq_index = trie.seq_index(1);
num_pos = data(1).num_pos;
num_neg = data(1).num_neg;

% if the rule is more than one word long, we need to see if the overlap
% window conditions are satisfied
if (rule_depth > 1)
    pos_counts = 0;
    neg_counts = 0;

    % loop through the test set and see which ones match
    % start with the positives
    for p = 1:num_pos
        s = rule.trie.seq_index(1);
        trie = tries(rule.features(1)).trie(s);
        last_starts = trie.pos_starts{p};
        last_feature = rule.features(1);

        for d = 2:rule_depth
            s = rule.trie.seq_index(d);
            feature = rule.features(d);
            trie = tries(feature).trie(s);
            starts = trie.pos_starts{p};

            valid_starts = [];
            for k = 1:length(last_starts)
                diffs = starts - last_starts(k);
                
                if (last_feature == feature)
                    seq_length = size(rule.trie.seq,2);
                    idxs = find(diffs >= (overlap_window + seq_length - 1) & diffs <= maximum_overlap);
                else
                    idxs = find(diffs >= overlap_window & diffs <= maximum_overlap);
                end
                
                valid_starts = [valid_starts, starts(idxs)];
            end
            last_starts = unique(valid_starts);
            last_feature = feature;
        end
        
        % if we have a non-empty set of valid starts, we can increment the
        % positive count
        if (~isempty(last_starts))
            disp(['true pos ', data(p).pos_name]);
            pos_counts = pos_counts + 1;
        end
    end
    
    % loop through the negatives
    for p = 1:num_neg
        s = rule.trie.seq_index(1);
        trie = tries(rule.features(1)).trie(s);
        last_starts = trie.neg_starts{p};
        last_feature = rule.features(1);

        for d = 2:rule_depth
            s = rule.trie.seq_index(d);
            feature = rule.features(d);
            trie = tries(feature).trie(s);
            starts = trie.neg_starts{p};

            valid_starts = [];
            for k = 1:length(last_starts)
                diffs = starts - last_starts(k);
                
                if (last_feature == feature)
                    seq_length = size(rule.trie.seq,2);
                    idxs = find(diffs >= (overlap_window + seq_length - 1) & diffs <= maximum_overlap);
                else
                    idxs = find(diffs >= overlap_window & diffs <= maximum_overlap);
                end
                
                valid_starts = [valid_starts, starts(idxs)];
            end
            last_starts = unique(valid_starts);
            last_feature = feature;
        end
        
        % if we have a non-empty set of valid starts, we can increment the
        % negative count
        if (~isempty(last_starts))
            neg_counts = neg_counts + 1;
            disp(['false pos ', data(p).neg_name]);
        end
    end
    

else
    % the rule is only of length 1 so we can use the counts in the trie
    pos_counts = tries(feature).trie(seq_index).pos_counts;
    neg_counts = tries(feature).trie(seq_index).neg_counts;
end

% Build the contingency table for this trie
table(1,1) = pos_counts;
table(1,2) = neg_counts;
table(2,1) = num_pos - pos_counts;
table(2,2) = num_neg - neg_counts;

% Calculate the Probability of Detection attained by the trie
pod = table(1,1) / (table(1,1) + table(2,1) + epsilon);

% Calculate the False Alarm Ratio attained by the trie
fa_ratio = table(1,2) / (table(1,1) + table(1,2) + epsilon);

% Calculate the False Alarm Rate attained by the trie
fa_rate = table(1,2) / (table(2,2) + table(1,2) + epsilon);

% Calculate the Critical Success Index attained by the trie
csi = table(1,1) / (table(1,1) + table(1,2) + table(2,1) + epsilon);

% save out the cross validated values
rule.pod_cross_val = pod;
rule.fa_ratio_cross_val = fa_ratio;
rule.fa_rate_cross_val = fa_rate;
rule.csi_cross_val = csi;
