% Copyright and terms of use (DO NOT REMOVE):
% The code is made freely available for non-commercial uses only, provided that the copyright 
% header in each file not be removed, and suitable citation(s) (see below) be made for papers 
% published based on the code.
%
% The code is not optimized for speed, and we are not responsible for any errors that might
% occur in the code.
%
% The copyright of the code is retained by the authors.  By downloading/using this code you
% agree to all the terms stated above.
%
%   Lin, J., Keogh, E., Lonardi, S. & Chiu, B. 
%   "A Symbolic Representation of Time Series, with Implications for Streaming Algorithms." 
%   In proceedings of the 8th ACM SIGMOD Workshop on Research Issues in Data Mining and 
%   Knowledge Discovery. San Diego, CA. June 13, 2003. 
%
%
%   Lin, J., Keogh, E., Patel, P. & Lonardi, S. 
%   "Finding Motifs in Time Series". In proceedings of the 2nd Workshop on Temporal Data Mining, 
%   at the 8th ACM SIGKDD International Conference on Knowledge Discovery and Data Mining. 
%   Edmonton, Alberta, Canada. July 23-26, 2002
%
% This function takes in a time series and convert it to string(s).
% There are two options:
%   1. Convert the entire time series to ONE string
%   2. Use sliding windows, extract the subsequences and convert these subsequences to strings
%
% For the first option, simply enter the length of the time series as "N"
%   ex. We have a time series of length 32 and we want to convert it to a 8-symbol string,
%       with alphabet size 3:
%       timeseries2symbol(data, 32, 8, 3)
% For the second option, enter the desired sliding window length as "N"
%   ex. We have a time series of length 32 and we want to extract subsequences of length 16 using
%       sliding windows, and convert the subsequences to 8-symbol strings, with alphabet size 3:
%       timeseries2symbol(data, 16, 8, 3)
% 
%
% Input:
%   data              is the raw time series. 
%   N                 is the length of sliding window (use the length of the raw time series
%                     instead if you don't want to have sliding windows)
%   n                 is the number of symbols in the low dimensional approximation of the sub sequence.
%   alphabet_size     is the number of discrete symbols. 2 <= alphabet_size <= 20, although 
%                     alphabet_size = 2 is a special "useless" case.
%
% Output:
%   symbolic_data:    matrix of symbolic data (no-repetition).  If consecutive subsequences
%                     have the same string, then only the first occurrence is recorded, with
%                     a pointer to its location stored in "pointers"
%   pointers:         location of the first occurrences of the strings
%
% The variable "win_size" is assigned to N/n, this is the number of data points on the raw 
% time series that will be mapped to a single symbol, and can be imagined as the 
% "compression rate".
%
% The symbolic data is returned in "symbolic_data", with pointers to the subsequences  
%
%
% 
%
% Copyright (c) 2003, Eamonn Keogh, Jessica Lin, Stefano Lonardi, Pranav Patel, Li Wei.  All rights reserved.
%



% *******************************begin******************************************
% Notes added by Meredith Beaton, 3/12/07
% All notes added by MB will be preceeded and concluded by ******
%
%
% data_matrix is expected to be a matrix of timeseries, such that each 
% column of the matrix is a feature of all storm data. 
%
% We return a struct called matrix_data that holds the symbolic_data and
% the pointers for each timeseries.
%
% We want to discretize the data based on the normalization of the data
% according to the original SAX algorithm. However, because we're dealing
% with multiple timeseries and we want compare the discretized values 
% across timeseries, we want to normalize with respect to all of the data. 
% That is, we need to find the mean and standard deviation for the entire 
% matrix. 
% *******************************end***************************************


function data =  timeseries2symbol_matrix(data, win_size, alphabet_size, pr)

load_feature_map;
global feature_map;

% ****************************begin**********************
% Flag if we want to print histograms of values
if(pr > 0 || pr <= 5)
    global counter_real_pos;
    global counter_dis_pos;
    global counter_real_neg;
    global counter_dis_neg;
    counter_real_pos = 1;
    counter_dis_pos = 1;
    counter_real_neg = 1;
    counter_dis_neg = 1;
    real_dir_pos = (['real_value_graphs_pos/']);
    real_dir_neg = (['real_value_graphs_neg/']);
    sax_dir_pos = (['sax_graphs_pos/']);
    sax_dir_neg = (['sax_graphs_neg/']);
    hist_dir = (['histograms/']);
end

% Put all of the data into a single matrix, storm_data
storm_data = [];
% get the positive storm data
for k = 1:data(1).num_pos
    % size(data(k).pos)
    storm_data = [storm_data; data(k).pos_pad'];
end

% get the negative storm data
for k = 1:data(1).num_neg
    storm_data = [storm_data; data(k).neg_pad'];
end
%****************************end**************************

if alphabet_size > 20
    disp('Currently alphabet_size cannot be larger than 20.  Please update the breakpoint table if you wish to do so');
    return;
end

% win_size is the number of data points on the raw time series that will be
% mapped to a single symbol
% win_size = floor(N/n);

pointers         = [];                                                  % Initialize pointers,
% symbolic_data = zeros(1,max_n);                                             % Initialize symbolic_data with a void string, it will be removed later.

% *********************************begin***********************************
[m p] = size(storm_data);
% ********************************end**************************************

% *********************************begin***********************************
% Iterate through each column of the matrix and treat each as an independent
% feature. Normalize with respect to the feature mean and the feature
% standard deviation.
for idx = 1 : p
    % Find the standard deviation and mean for the entire feature
    % remove any padded NaNs for irregular length data
    valid_idxs = ~isnan(storm_data(:,idx));

    % Mean for the entire data matrix
    data_mean = mean(storm_data(valid_idxs,idx));

    % Standard deviation for the data matrix
    data_std = std(storm_data(valid_idxs,idx));

    feature_map(idx).mean = data_mean;
    feature_map(idx).std = data_std;
    
    %% Print out histograms of values across all storms
    if(pr == 1 || pr == 5)
        print_histograms(idx, feature_map, storm_data, hist_dir);
    end


    % Get the positive & negative storms data for feature idx
    for k = 1:(data(1).num_pos+data(1).num_neg)
        if(k <= data(1).num_pos)
            sax_data = data(k).pos(idx,:);
        else
            sax_data = data(k-data(1).num_pos).neg(idx,:);
        end
        
        
        
        if(k == 5 || k == 73)
            % Print the discretized data
            if(pr  == 2 || pr == 4 || pr == 5)
                if(k <= data(1).num_pos)
                    my_dir = (['sax_graphs_pos/']);
                    print_discretized(idx, feature_map, sax_data, 1, ...
                        data_mean, data_std, data(k).pos_name, sax_dir_pos);
                else
                    my_dir = (['sax_graphs_neg/']);
                    print_discretized(idx, feature_map, sax_data, 0, ...
                        data_mean, data_std, data(k).neg_name, sax_dir_neg);
                end


            end
            
            % Print the real-valued data
            if(pr == 3 || pr == 4 || pr == 5)
                if(k <= data(1).num_pos)
                    my_dir = (['real_value_graphs_pos/']);
                    print_real_values(idx, feature_map, sax_data, 1, ...
                        data(k).pos_name, real_dir_pos);
                else
                    my_dir = (['real_value_graphs_neg/']);
                    print_real_values(idx, feature_map, sax_data, 0, ...
                        data(k).neg_name, real_dir_neg);
                end
            end
        end
        % zero out pointers and symbolic data before starting on a new
        % storm
        pointers         = [];

        % *********** added by Amy McGovern 08/27/09
        % this makes the sliding window not appear for
        % our uneven time series data
        N = length(sax_data);
        n = floor(N/win_size);
        symbolic_data = zeros(1,n);
        % *********** end
        
        % ********************************end**************************************
        % Scan accross the time series extract sub sequences, and converting them to strings.
        for i = 1 : length(sax_data) - (N -1)

            % Remove the current subsection.
            sub_section = sax_data(i:i + N -1);

            % Z normalize it.
            % *******************************begin*****************************
            % normalize with respect to the statistics for the entire matrix
            if(data_std > 0)
                sub_section = (sub_section - data_mean)/data_std;
            end
            % *******************************end*******************************

            % take care of the special case where there is no dimensionality reduction
            if N == n
                PAA = sub_section;

                % Convert to PAA.
            else
                % N is not dividable by n
                if (N/n - floor(N/n) ~= 0)
                    temp = zeros(n, N);
                    for j = 1 : n
                        temp(j, :) = sub_section;
                    end
                    expanded_sub_section = reshape(temp, 1, N*n);
                    PAA = [mean(reshape(expanded_sub_section, N, n))];
                    % N is dividable by n
                else
                    PAA = [mean(reshape(sub_section,win_size,n))];
                end
            end

            current_string = map_to_string(PAA,alphabet_size);          % Convert the PAA to a string.

            if (~all(current_string == symbolic_data(end,:)))             % If the string differs from its leftmost neighbor...
                symbolic_data    = [symbolic_data; current_string];     % ... add it to the set...
                pointers         = [pointers ; i];                      % ... and add a new pointer.
            end;
        end;
        % Delete the first element, it was just used to initialize the data structure
        symbolic_data(1,:) = [];
        
        % **********************************begin******************************
        % For our purposes of making Tries later, we need symbolic_data to be
        % in the form of a single vector. Currently, symbolic_data can be a
        % matrix
        [x z] = size(symbolic_data);
        if(x>1)
            new_sd = [];
            for z = 1:x
                g = symbolic_data(z,:);
                new_sd = [new_sd, g];
            end
            symbolic_data = new_sd;
        end

        % Save all saxed storm data into data
        if(k <= data(1).num_pos)
            data(k).discrete_pos(idx).symbols = symbolic_data;
            data(k).discrete_pos(idx).pointers = pointers;
            data(k).discrete_pos(idx).PAA = PAA;
        else
            data(k-data(1).num_pos).discrete_neg(idx).symbols = symbolic_data;
            data(k-data(1).num_pos).discrete_neg(idx).pointers = pointers;
            data(k-data(1).num_pos).discrete_neg(idx).PAA = PAA;
        end
        
    end
    %**********************************end*********************************
end

                                           

%--------------------------------------------------------------------------------------------------------------------------------------------------------
%----------------Local Functions----------------------Local Functions----------------Local Functions----------------------Local Functions----------------
%--------------------------------------------------------------------------------------------------------------------------------------------------------


function string = map_to_string(PAA,alphabet_size)

string = zeros(1,length(PAA));

switch alphabet_size
    case 2, cut_points  = [-inf 0];
    case 3, cut_points  = [-inf -0.43 0.43];
    case 4, cut_points  = [-inf -0.67 0 0.67];
    case 5, cut_points  = [-inf -0.84 -0.25 0.25 0.84];
    case 6, cut_points  = [-inf -0.97 -0.43 0 0.43 0.97];
    case 7, cut_points  = [-inf -1.07 -0.57 -0.18 0.18 0.57 1.07];
    case 8, cut_points  = [-inf -1.15 -0.67 -0.32 0 0.32 0.67 1.15];
    case 9, cut_points  = [-inf -1.22 -0.76 -0.43 -0.14 0.14 0.43 0.76 1.22];
    case 10, cut_points = [-inf -1.28 -0.84 -0.52 -0.25 0. 0.25 0.52 0.84 1.28];
    case 11, cut_points = [-inf -1.34 -0.91 -0.6 -0.35 -0.11 0.11 0.35 0.6 0.91 1.34];
    case 12, cut_points = [-inf -1.38 -0.97 -0.67 -0.43 -0.21 0 0.21 0.43 0.67 0.97 1.38];
    case 13, cut_points = [-inf -1.43 -1.02 -0.74 -0.5 -0.29 -0.1 0.1 0.29 0.5 0.74 1.02 1.43];
    case 14, cut_points = [-inf -1.47 -1.07 -0.79 -0.57 -0.37 -0.18 0 0.18 0.37 0.57 0.79 1.07 1.47];
    case 15, cut_points = [-inf -1.5 -1.11 -0.84 -0.62 -0.43 -0.25 -0.08 0.08 0.25 0.43 0.62 0.84 1.11 1.5];
    case 16, cut_points = [-inf -1.53 -1.15 -0.89 -0.67 -0.49 -0.32 -0.16 0 0.16 0.32 0.49 0.67 0.89 1.15 1.53];
    case 17, cut_points = [-inf -1.56 -1.19 -0.93 -0.72 -0.54 -0.38 -0.22 -0.07 0.07 0.22 0.38 0.54 0.72 0.93 1.19 1.56];
    case 18, cut_points = [-inf -1.59 -1.22 -0.97 -0.76 -0.59 -0.43 -0.28 -0.14 0 0.14 0.28 0.43 0.59 0.76 0.97 1.22 1.59];
    case 19, cut_points = [-inf -1.62 -1.25 -1 -0.8 -0.63 -0.48 -0.34 -0.2 -0.07 0.07 0.2 0.34 0.48 0.63 0.8 1 1.25 1.62];
    case 20, cut_points = [-inf -1.64 -1.28 -1.04 -0.84 -0.67 -0.52 -0.39 -0.25 -0.13 0 0.13 0.25 0.39 0.52 0.67 0.84 1.04 1.28 1.64];
    otherwise disp('Error! alphabet_size is too big');
end;

for i = 1 : length(PAA)
    string(i) = sum( (cut_points <= PAA(i)), 2 );         % order is now: a = 1, b = 2, c = 3..
end;


% *********************** begin ******************************
function print_histograms(idx, feature_map, storm_data, hist_dir)
    figure;
    hist(storm_data(:,idx), 20);
    title(['Histogram of values across all storms for ', feature_map(idx).name]);

    if((idx >= 1 && idx <= 6) || (idx >=44 && idx <= 46))
        xlabel('m s^{-1}');
    elseif((idx >= 7 && idx <= 14) || (idx >= 32 && idx <= 35) || (idx >= 44 && idx == 58))
        xlabel('s^{-1}');
    elseif((idx == 15 || idx == 16) || (idx ==36  || idx == 37))
        xlabel('kg kg^{-1}');
    elseif((idx >= 17 && idx <= 22) || (idx >= 38  || idx <= 43))
        xlabel('kg kg^{-1} m^{-1}');
    elseif(idx == 23 || idx == 24)
        xlabel('K');
    elseif(idx == 21 || idx == 22 || idx == 23)
        xlabel('Pascal');
    elseif(idx >= 25 && idx <= 27)
        xlabel('m s^{-2}');
    elseif(idx == 66 || idx == 67)
        xlabel('K m^{-1}');
    elseif(idx == 68 || idx == 69)
        xlabel('dBZ');
    elseif(idx >= 70 && idx <= 75)
        xlabel('dBZ m^{-1}');
    end
    ylabel('counts');
    fileName = [hist_dir, 'hist_', num2str(idx), '.eps'];
    print('-depsc ', fileName);
    close all;


function print_discretized(idx, feature_map, dis_data, pos_neg, ...
        data_mean, data_std, storm_name, dis_dir)
    global counter_dis_pos;
    global counter_dis_neg;
    print_sax(dis_data, data_mean, data_std)
    title({['Discretized data for ', feature_map(idx).name]; ...
        ['for storm ', storm_name]});
    xlabel('Time (where each interval is 30 seconds)');
    
    if(pos_neg == 1)
        fileName = [dis_dir, 'discretized_data_', num2str(counter_dis_pos),'.eps'];
        counter_dis_pos = counter_dis_pos + 1;
    else
        fileName = [dis_dir, 'discretized_data_', num2str(counter_dis_neg),'.eps'];
        counter_dis_neg = counter_dis_neg + 1;
    end


    if((idx >= 1 && idx <= 6) || (idx >=44 && idx <= 46))
        ylabel('m s^{-1}');
    elseif((idx >= 7 && idx <= 14) || (idx >= 32 && idx <= 35) || (idx >= 44 && idx == 58))
        ylabel('s^{-1}');
    elseif((idx == 15 || idx == 16) || (idx ==36  || idx == 37))
        ylabel('kg kg^{-1}');
    elseif((idx >= 17 && idx <= 22) || (idx >= 38  || idx <= 43))
        ylabel('kg kg^{-1} m^{-1}');
    elseif(idx == 23 || idx == 24)
        ylabel('K');
    elseif(idx == 21 || idx == 22 || idx == 23)
        ylabel('Pascal');
    elseif(idx >= 25 && idx <= 27)
        ylabel('m s^{-2}');
    elseif(idx == 66 || idx == 67)
        ylabel('K m^{-1}');
    elseif(idx == 68 || idx == 69)
        ylabel('dBZ');
    elseif(idx >= 70 && idx <= 75)
        ylabel('dBZ m^{-1}');
    end

    print('-depsc ',fileName);

    close all;

function print_real_values(idx, feature_map, real_data, pos_neg, storm_name, real_dir)
    global counter_real_pos;
    global counter_real_neg;

    h = plot(real_data);
    set(h, 'LineWidth', 2);
    title({['Real valued data for ', feature_map(idx).name]; ...
        ['for storm ', storm_name]});
    xlabel('Time (where each interval is 30 seconds)');
    
    if(pos_neg == 1)
        fileName = [real_dir, 'real_valued_data_', num2str(counter_real_pos),'.eps'];
        counter_real_pos = counter_real_pos + 1;
    else
        fileName = [real_dir, 'real_valued_data_', num2str(counter_real_neg),'.eps'];
        counter_real_neg = counter_real_neg + 1;
    end

    if((idx >= 1 && idx <= 6) || (idx >=44 && idx <= 46))
        ylabel('m s^{-1}');
    elseif((idx >= 7 && idx <= 14) || (idx >= 32 && idx <= 35) || ...
            (idx >= 44 && idx == 58))
        ylabel('s^{-1}');
    elseif((idx == 15 || idx == 16) || (idx ==36  || idx == 37))
        ylabel('kg kg^{-1}');
    elseif((idx >= 17 && idx <= 22) || (idx >= 38  || idx <= 43))
        ylabel('kg kg^{-1} m^{-1}');
    elseif(idx == 23 || idx == 24)
        ylabel('K');
    elseif(idx == 21 || idx == 22 || idx == 23)
        ylabel('Pascal');
    elseif(idx >= 25 && idx <= 27)
        ylabel('m s^{-2}');
    elseif(idx == 66 || idx == 67)
        ylabel('K m^{-1}');
    elseif(idx == 68 || idx == 69)
        ylabel('dBZ');
    elseif(idx >= 70 && idx <= 75)
        ylabel('dBZ m^{-1}');
    end

    print('-depsc ',fileName);

    close all;


